import 'dart:math';

typedef Aggegator<I, O> = O Function(Iterable<I> values);

N sum<N extends num>(Iterable<N> values) => values.reduce((N value1, N value2) => (value1 + value2) as N);

double average<N extends num>(Iterable<N> values) => sum(values) / values.length;

double standardDeviation(Iterable<num> values) {
  var avg = average(values);
  return sqrt(sum(values.map((value) => pow(value - avg, 2))) / values.length - 1);
}

DataPoint averageAndStandardDeviation(Iterable<num> values) => DataPoint(average(values), standardDeviation(values));

N minimum<N extends num>(Iterable<N> values) => values.reduce(min);

N maximum<N extends num>(Iterable<N> values) => values.reduce(max);

class DataPoint {
  final num average;
  final num standardDeviation;

  DataPoint(this.average, this.standardDeviation);

  DataPoint.fromJson(Map json) : this(json['average'], json['standardDeviation']);

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is DataPoint &&
          runtimeType == other.runtimeType &&
          average == other.average &&
          standardDeviation == other.standardDeviation;

  @override
  int get hashCode => average.hashCode ^ standardDeviation.hashCode;

  @override
  String toString() {
    return '$average (±$standardDeviation)';
  }

  Map<String, num> toJson() => {
        'average': average,
        'standardDeviation': standardDeviation,
      };
}
