import 'package:test/test.dart';
import 'package:time_series/time_series.dart';

void main() {
  test('points', () {
    var mw = Unit('Megawatt', 'MW');
    var timeSeries = TimeSeries(
      start: DateTime.parse('2020-10-01T00:00:00'),
      timeUnit: TimeUnit.hour,
      unit: mw,
      values: [370, 384, 412],
    );

    expect(timeSeries.points, {
      DateTime.parse('2020-10-01T00:00:00'): Quantity(mw, 370),
      DateTime.parse('2020-10-01T01:00:00'): Quantity(mw, 384),
      DateTime.parse('2020-10-01T02:00:00'): Quantity(mw, 412),
    });
  });

  test('aggregate', () {
    var mw = Unit('Megawatt', 'MW');
    var timeSeries = TimeSeries(
      start: DateTime.parse('2019-01-01T00:00:00'),
      timeUnit: TimeUnit.month,
      unit: mw,
      values: [370, 384, 413, 435, 443, 439, 419, 503, 470, 456, 435, 432, 427, 428, 423, 410, 396, 382],
    );

    var aggregated = timeSeries.aggregate(TimeUnit.quarter, average);

    expect(aggregated.timeUnit, TimeUnit.quarter);
    expect(aggregated.points, {
      DateTime.parse('2019-01-01T00:00:00'): Quantity(mw, 389.0),
      DateTime.parse('2019-04-01T00:00:00'): Quantity(mw, 439.0),
      DateTime.parse('2019-07-01T00:00:00'): Quantity(mw, 464.0),
      DateTime.parse('2019-10-01T00:00:00'): Quantity(mw, 441.0),
      DateTime.parse('2020-01-01T00:00:00'): Quantity(mw, 426.0),
      DateTime.parse('2020-04-01T00:00:00'): Quantity(mw, 396.0),
    });
  });

  test('merge', () {
    var mw = Unit('Megawatt', 'MW');
    var timeSeries1 = TimeSeries(
      start: DateTime.parse('2020-10-01T00:00:00'),
      timeUnit: TimeUnit.hour,
      unit: mw,
      values: [370, 384, 412],
    );
    var timeSeries2 = TimeSeries(
      start: DateTime.parse('2020-10-01T04:00:00'),
      timeUnit: TimeUnit.hour,
      unit: mw,
      values: [444, 439, 210],
    );
    var timeSeries3 = TimeSeries(
      start: DateTime.parse('2020-10-01T06:00:00'),
      timeUnit: TimeUnit.hour,
      unit: mw,
      values: [302, 469, 457],
    );

    var merge = timeSeries1.merge(timeSeries2.merge(timeSeries3));

    expect(merge.points, {
      DateTime.parse('2020-10-01T00:00:00'): Quantity(mw, 370),
      DateTime.parse('2020-10-01T01:00:00'): Quantity(mw, 384),
      DateTime.parse('2020-10-01T02:00:00'): Quantity(mw, 412),
      DateTime.parse('2020-10-01T03:00:00'): Quantity(mw, 0),
      DateTime.parse('2020-10-01T04:00:00'): Quantity(mw, 444),
      DateTime.parse('2020-10-01T05:00:00'): Quantity(mw, 439),
      DateTime.parse('2020-10-01T06:00:00'): Quantity(mw, 512),
      DateTime.parse('2020-10-01T07:00:00'): Quantity(mw, 469),
      DateTime.parse('2020-10-01T08:00:00'): Quantity(mw, 457),
    });
  });
}
